@@ -14,7 +14,7 @@
 ** but WITHOUT ANY WARRANTY; without even the implied warranty of
 ** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 ** Library General Public License for more details.
-** 
+**
 ** You should have received a copy of the GNU Library General Public
 ** License along with the NYS Library; see the file COPYING.LIB.  If
 ** not, write to the Free Software Foundation, Inc., 675 Mass Ave,
@@ -56,13 +56,11 @@
 ** SUCH DAMAGE.
 */
 
-static char rcsid[] = "$Id: dns_hosts.c,v 1.0.4.1 1996/07/14 15:10:45 kukuk Exp $";
-
 #if defined(HAVE_CONFIG_H)
 #include "config.h"
 #endif
 
-#if defined(linux) && !defined(HAVE__DNS_GETHOSTBYNAME)
+#if defined(linux) && !defined(HAVE__DNS_GETHOSTBYNAME) && !defined(HAVE_RES_GETHOSTBYNAME)
 
 #include <sys/param.h>
 #include <sys/socket.h>
@@ -357,7 +355,7 @@ addrsort(char **ap, int num)
 	p = ap;
 	for (i = 0; i < num; i++, p++) {
 	    for (j = 0 ; (unsigned)j < _res.nsort; j++)
-		if (_res.sort_list[j].addr.s_addr == 
+		if (_res.sort_list[j].addr.s_addr ==
 		    (((struct in_addr *)(*p))->s_addr & _res.sort_list[j].mask))
 			break;
 	    aval[i] = j;
@@ -432,7 +430,7 @@ _dns_gethostbyname(const char *name)
 				h_errno = NETDB_SUCCESS;
 				return (&host);
 			}
-			if (!isdigit(*cp) && *cp != '.') 
+			if (!isdigit(*cp) && *cp != '.')
 				break;
 		}
 
@@ -456,7 +454,7 @@ _dns_gethostbyaddr(const char *addr, int len, int type)
 	u_long old_options;
 	char hname2[MAXDNAME+1];
 #endif /*SUNSECURITY*/
-	
+
 	if (type != AF_INET) {
 		errno = EAFNOSUPPORT;
 		h_errno = NETDB_INTERNAL;
