@@ -1,7 +1,7 @@
 /*
 ** ypserv.c - YP server main routines.
 **
-** Copyright (c) 1996, 1997, 1998 Thorsten Kukuk
+** Copyright (c) 1996, 1997, 1998, 1999 Thorsten Kukuk
 ** Copyright (c) 1993 Signum Support AB, Sweden
 **
 ** This file is part of the NYS YP Server.
@@ -22,13 +22,9 @@
 ** Cambridge, MA 02139, USA.
 **
 ** Author: Peter Eriksson <pen@signum.se>
-** Modified: Thorsten Kukuk <kukuk@uni-paderborn.de>
+** Modified: Thorsten Kukuk <kukuk@suse.de>
 */
 
-#ifndef LINT
-static const char rcsid[] = "$Id: ypserv.c,v 1.10 1998/01/09 20:41:43 kukuk Exp $";
-#endif
-
 #ifdef HAVE_CONFIG_H
 #include "config.h"
 #endif
@@ -87,7 +83,6 @@ static const char rcsid[] = "$Id: ypserv.c,v 1.10 1998/01/09 20:41:43 kukuk Exp
 int version_flag = 0;
 int debug_flag = 0;
 int dns_flag = 0;
-int sunos_kludge_flag = 1;
 
 #ifndef YPMAPDIR
 #define YPMAPDIR   "/var/yp"
@@ -131,6 +126,105 @@ int _rpc_dtablesize()
 #define SIG_PF void(*)(int)
 #endif
 
+static void
+ypprog_1 (struct svc_req *rqstp, register SVCXPRT *transp)
+{
+  union {
+    domainname ypoldproc_domain_1_arg;
+    domainname ypoldproc_domain_nonack_1_arg;
+    yprequest ypoldproc_match_1_arg;
+    yprequest ypoldproc_first_1_arg;
+    yprequest ypoldproc_next_1_arg;
+    yprequest ypoldproc_poll_1_arg;
+    yprequest ypoldproc_push_1_arg;
+    yprequest ypoldproc_pull_1_arg;
+    yprequest ypoldproc_get_1_arg;
+  } argument;
+  char *result;
+  xdrproc_t xdr_argument, xdr_result;
+  char *(*local)(char *, struct svc_req *);
+
+  switch (rqstp->rq_proc) {
+  case YPOLDPROC_NULL:
+    xdr_argument = (xdrproc_t) xdr_void;
+    xdr_result = (xdrproc_t) xdr_void;
+    local = (char *(*)(char *, struct svc_req *)) ypoldproc_null_1_svc;
+    break;
+
+  case YPOLDPROC_DOMAIN:
+    xdr_argument = (xdrproc_t) xdr_domainname;
+    xdr_result = (xdrproc_t) xdr_bool;
+    local = (char *(*)(char *, struct svc_req *)) ypoldproc_domain_1_svc;
+    break;
+
+  case YPOLDPROC_DOMAIN_NONACK:
+    xdr_argument = (xdrproc_t) xdr_domainname;
+    xdr_result = (xdrproc_t) xdr_bool;
+    local = (char *(*)(char *, struct svc_req *)) ypoldproc_domain_nonack_1_svc;
+    break;
+
+  case YPOLDPROC_MATCH:
+    xdr_argument = (xdrproc_t) xdr_yprequest;
+    xdr_result = (xdrproc_t) xdr_ypresponse;
+    local = (char *(*)(char *, struct svc_req *)) ypoldproc_match_1_svc;
+    break;
+
+  case YPOLDPROC_FIRST:
+    xdr_argument = (xdrproc_t) xdr_yprequest;
+    xdr_result = (xdrproc_t) xdr_ypresponse;
+    local = (char *(*)(char *, struct svc_req *)) ypoldproc_first_1_svc;
+    break;
+
+  case YPOLDPROC_NEXT:
+    xdr_argument = (xdrproc_t) xdr_yprequest;
+    xdr_result = (xdrproc_t) xdr_ypresponse;
+    local = (char *(*)(char *, struct svc_req *)) ypoldproc_next_1_svc;
+    break;
+
+  case YPOLDPROC_POLL:
+    xdr_argument = (xdrproc_t) xdr_yprequest;
+    xdr_result = (xdrproc_t) xdr_ypresponse;
+    local = (char *(*)(char *, struct svc_req *)) ypoldproc_poll_1_svc;
+    break;
+
+  case YPOLDPROC_PUSH:
+    xdr_argument = (xdrproc_t) xdr_yprequest;
+    xdr_result = (xdrproc_t) xdr_ypresponse;
+    local = (char *(*)(char *, struct svc_req *)) ypoldproc_push_1_svc;
+    break;
+
+  case YPOLDPROC_PULL:
+    xdr_argument = (xdrproc_t) xdr_yprequest;
+    xdr_result = (xdrproc_t) xdr_ypresponse;
+    local = (char *(*)(char *, struct svc_req *)) ypoldproc_pull_1_svc;
+    break;
+
+  case YPOLDPROC_GET:
+    xdr_argument = (xdrproc_t) xdr_yprequest;
+    xdr_result = (xdrproc_t) xdr_ypresponse;
+    local = (char *(*)(char *, struct svc_req *)) ypoldproc_get_1_svc;
+    break;
+
+  default:
+    svcerr_noproc (transp);
+    return;
+  }
+  memset ((char *)&argument, 0, sizeof (argument));
+  if (!svc_getargs (transp, xdr_argument, (caddr_t) &argument)) {
+    svcerr_decode (transp);
+    return;
+  }
+  result = (*local)((char *)&argument, rqstp);
+  if (result != NULL && !svc_sendreply(transp, xdr_result, result)) {
+    svcerr_systemerr (transp);
+  }
+  if (!svc_freeargs (transp, xdr_argument, (caddr_t) &argument)) {
+    fprintf (stderr, "unable to free arguments");
+    exit (1);
+  }
+  return;
+}
+
 static void
 ypprog_2(struct svc_req *rqstp, register SVCXPRT *transp)
 {
@@ -254,49 +348,47 @@ ypprog_2(struct svc_req *rqstp, register SVCXPRT *transp)
   return;
 }
 
-void ypserv_svc_run(void)
+extern int forked;
+
+void
+ypserv_svc_run (void)
 {
 #ifdef FD_SETSIZE
   fd_set readfds;
 #else
   int readfds;
 #endif /* def FD_SETSIZE */
-  extern int forked;
-  int pid;
 
-  /* Establish the identity of the parent ypserv process. */
-  pid = getpid();
-
-  for (;;) {
+  for (;;)
+    {
 #ifdef FD_SETSIZE
-    readfds = svc_fdset;
+      readfds = svc_fdset;
 #else
-    readfds = svc_fds;
+      readfds = svc_fds;
 #endif /* def FD_SETSIZE */
-    switch (select(_rpc_dtablesize(), &readfds, NULL, NULL,
-		   (struct timeval *)0))
-      {
-      case -1:
-	if (errno == EINTR)
+      switch (select(_rpc_dtablesize(), &readfds, NULL, NULL,
+		     (struct timeval *)0))
+	{
+	case -1:
+	  if (errno == EINTR)
 	    continue;
-	syslog(LOG_ERR, "svc_run: - select failed: %m");
-	return;
-      case 0:
-	continue;
-      default:
-	svc_getreqset(&readfds);
-	if (forked && pid != getpid())
-	  exit(0);
-      }
-  }
+	  syslog (LOG_ERR, "svc_run: - select failed: %m");
+	  return;
+	case 0:
+	  continue;
+	default:
+	  svc_getreqset (&readfds);
+	  if (forked)
+	    exit (0);
+	}
+    }
 }
 
-/*
-** Create a pidfile on startup
-*/
-void create_pidfile(void)
+/* Create a pidfile on startup */
+static void
+create_pidfile(void)
 {
-  int fd, left, written;
+  int fd, left, written, flags;
   pid_t pid;
   char pbuf[10], *ptr;
   struct flock lock;
@@ -309,6 +401,22 @@ void create_pidfile(void)
       if(debug_flag) yp_msg("\n");
     }
 
+  /* Make sure file gets correctly closed when process finished.  */
+  flags = fcntl (fd, F_GETFD, 0);
+  if (flags == -1)
+    {
+      /* Cannot get file flags.  */
+      close (fd);
+      return;
+    }
+  flags |= FD_CLOEXEC;          /* Close on exit.  */
+  if (fcntl (fd, F_SETFD, flags) < 0)
+    {
+      /* Cannot set new flags.  */
+      close (fd);
+      return;
+    }
+
   lock.l_type = F_WRLCK;
   lock.l_start = 0;
   lock.l_whence = SEEK_SET;
@@ -353,75 +461,61 @@ void create_pidfile(void)
   return;
 }
 
-/*
-** Clean up after child processes signal their termination.
-*/
-void sig_child(int sig)
+/* Clean up after child processes signal their termination. */
+static void
+sig_child (int sig __attribute__ ((unused)))
 {
-   int st;
+  int st;
 
 #ifdef HAVE_WAIT3
-   while (wait3(&st, WNOHANG, NULL) > 0)
+  while (wait3 (&st, WNOHANG, NULL) > 0)
+    --children;
 #else
-   while (waitpid(-1, &st, WNOHANG) > 0)
+  while (waitpid (-1, &st, WNOHANG) > 0)
+    --children;
 #endif
-      children--;
-}
-
-/*
-** Clean up if we quit the program.
-*/
-void sig_quit(int sig)
-{
-  (void) pmap_unset(YPPROG, YPVERS);
-  if(sunos_kludge_flag)
-    (void) pmap_unset(YPPROG, 1);
-  unlink(_YPSERV_PIDFILE);
-
-  exit(0);
-}
-
-static SVCXPRT    *main_transp;
-/*
-** Reload securenets and config file
-*/
-void sig_hup(int sig)
-{
-  int old_sunos_kludge;
-
-  old_sunos_kludge = sunos_kludge_flag;
-
-#ifndef HAVE_LIBWRAP
-  load_securenets();
-#endif
-  load_config();
-
-  if(sunos_kludge_flag && ! old_sunos_kludge)
-    {/* We must register it new */
-      if (!svc_register(main_transp, YPPROG, 1, ypprog_2, IPPROTO_UDP))
-	{
-	  yp_msg("unable to register (YPPROG, 1, udp).");
-	  sunos_kludge_flag = 0;
-	}
-    }
+#ifdef DEBUG
+  if (children < 0)
+    yp_msg ("children is lower 0!\n");
   else
-    if(!sunos_kludge_flag && old_sunos_kludge)
-      { /* We must unregister it now */
-	(void) pmap_unset(YPPROG, 1);
-      }
+    yp_msg ("children = %i\n", children);
+#endif
 }
 
-void Usage(int exitcode)
+/* Clean up if we quit the program. */
+static void
+sig_quit (int sig __attribute__ ((unused)))
+{
+  pmap_unset (YPPROG, YPVERS);
+  pmap_unset (YPPROG, YPOLDVERS);
+  unlink (_YPSERV_PIDFILE);
+
+  exit (0);
+}
+
+/* Reload securenets and config file */
+static void
+sig_hup (int sig __attribute__ ((unused)))
+{
+#ifndef HAVE_LIBWRAP
+  load_securenets ();
+#endif
+  load_config ();
+}
+
+static void
+Usage (int exitcode)
 {
   fputs ("usage: ypserv [-d [path]] [-b] [-p port]\n", stderr);
   fputs ("       ypserv --version\n", stderr);
 
-  exit(exitcode);
+  exit (exitcode);
 }
 
 int
 main (int argc, char **argv)
 {
+  SVCXPRT *main_transp;
   int my_port = -1, my_socket, result;
   struct sockaddr_in socket_address;
   struct sigaction sa;
@@ -456,7 +550,7 @@ main (int argc, char **argv)
         case 'v':
 	  debug_flag=1;
 	  yp_msg("ypserv - NYS YP Server version %s (with %s)\n", version,
-#if HAVE_LIBWRAP
+#ifdef HAVE_LIBWRAP
 		 "tcp wrapper"
 #else
 		 "securenets"
@@ -490,7 +584,7 @@ main (int argc, char **argv)
   if (debug_flag)
     yp_msg("[Welcome to the NYS YP Server, version %s (with %s)]\n",
 	   version,
-#if HAVE_LIBWRAP
+#ifdef HAVE_LIBWRAP
 	   "tcp wrapper"
 #else
 	   "securenets"
@@ -500,14 +594,38 @@ main (int argc, char **argv)
     {
       int i;
 
-      if (fork())
+      if ((i = fork()) > 0)
 	exit(0);
 
-      for (i = 0; i < 255; i++)
+      if (i < 0)
+	{
+	  yp_msg ("Cannot fork: %s\n", strerror (errno));
+	  exit (-1);
+	}
+
+      if (setsid() == -1)
+	{
+	  yp_msg ("Cannot setsid: %s\n", strerror (errno));
+	  exit (-1);
+	}
+
+      if ((i = fork()) > 0)
+	exit(0);
+
+      if (i < 0)
+	{
+	  yp_msg ("Cannot fork: %s\n", strerror (errno));
+	  exit (-1);
+	}
+
+      for (i = 0; i < getdtablesize(); ++i)
 	close(i);
+      errno = 0;
 
-      if (fork())
-	exit(0);
+      umask(0);
+      i = open("/dev/null", O_RDWR);
+      dup(i);
+      dup(i);
     }
 
   if (argc > 0)
@@ -587,9 +705,8 @@ main (int argc, char **argv)
   sigemptyset(&sa.sa_mask);
   sigaction(SIGHUP, &sa, NULL);
 
-  (void) pmap_unset(YPPROG, YPVERS);
-  if(sunos_kludge_flag)
-    (void) pmap_unset(YPPROG, 1);
+  pmap_unset(YPPROG, YPVERS);
+  pmap_unset(YPPROG, YPOLDVERS);
 
   if (my_port >= 0)
     {
@@ -617,41 +734,35 @@ main (int argc, char **argv)
   else
     my_socket = RPC_ANYSOCK;
 
-  main_transp = svcudp_create(my_socket);
-  if (main_transp == NULL) {
-    yp_msg("cannot create udp service.");
-    exit(1);
-  }
-  if (!svc_register(main_transp, YPPROG, YPVERS, ypprog_2, IPPROTO_UDP)) {
-    yp_msg("unable to register (YPPROG, YPVERS, udp).");
-    exit(1);
-  }
+  main_transp = svcudp_create (my_socket);
+  if (main_transp == NULL)
+    {
+      yp_msg ("cannot create udp service.");
+      exit (1);
+    }
 
-  /*
-  ** This is just to make us reply to YP version 1 calls which SunOS 4's
-  ** ypbind seems to insist on finding. If someone _really_ tries to
-  ** use this then they will probably be bitten - _hard_, since I haven't
-  ** got the faintest idea on how the XDR calls for YP version 1 should
-  ** look like. The Domain_NoNack call seems to be compatible though :-)
-  */
-  if(sunos_kludge_flag)
-    if (!svc_register(main_transp, YPPROG, 1, ypprog_2, IPPROTO_UDP))
-      {
-	yp_msg("unable to register (YPPROG, 1, udp).");
-	exit(1);
-      }
+  if (!svc_register (main_transp, YPPROG, YPVERS, ypprog_2, IPPROTO_UDP))
+    {
+      yp_msg ("unable to register (YPPROG, YPVERS, udp).");
+      exit (1);
+    }
+
+  if (!svc_register(main_transp, YPPROG, YPOLDVERS, ypprog_1, IPPROTO_UDP))
+    {
+      yp_msg("unable to register (YPPROG, YPOLDVERS, udp).");
+      exit(1);
+    }
 
   if (my_port >= 0)
     {
       my_socket = socket (AF_INET, SOCK_STREAM, 0);
       if (my_socket < 0)
 	{
-	  yp_msg ("%s: can not create TCP ",
-		  progname);
+	  yp_msg ("%s: can not create TCP: %s", progname, strerror (errno));
 	  exit (1);
 	}
 
-      memset((char *) &socket_address, 0, sizeof(socket_address));
+      memset (&socket_address, 0, sizeof (socket_address));
       socket_address.sin_family = AF_INET;
       socket_address.sin_addr.s_addr = htonl (INADDR_ANY);
       socket_address.sin_port = htons (my_port);
@@ -660,8 +771,7 @@ main (int argc, char **argv)
 		     sizeof (socket_address));
       if (result < 0)
 	{
-	  yp_msg("%s: can not bind TCP ",
-		 progname);
+	  yp_msg("%s: can not bind TCP ", progname, strerror (errno));
 	  exit (1);
 	}
     }
@@ -669,14 +779,23 @@ main (int argc, char **argv)
     my_socket = RPC_ANYSOCK;
 
   main_transp = svctcp_create(my_socket, 0, 0);
-  if (main_transp == NULL) {
-    yp_msg("%s: cannot create tcp service\n", progname);
-    exit(1);
-  }
-  if (!svc_register(main_transp, YPPROG, YPVERS, ypprog_2, IPPROTO_TCP)) {
-    yp_msg("%s: unable to register (YPPROG, YPVERS, tcp)\n", progname);
-    exit(1);
-  }
+  if (main_transp == NULL)
+    {
+      yp_msg("%s: cannot create tcp service\n", progname);
+      exit(1);
+    }
+
+  if (!svc_register(main_transp, YPPROG, YPVERS, ypprog_2, IPPROTO_TCP))
+    {
+      yp_msg("%s: unable to register (YPPROG, YPVERS, tcp)\n", progname);
+      exit(1);
+    }
+
+  if (!svc_register(main_transp, YPPROG, YPOLDVERS, ypprog_1, IPPROTO_TCP))
+    {
+      yp_msg("%s: unable to register (YPPROG, YPOLDVERS, tcp)\n", progname);
+      exit(1);
+    }
 
   ypserv_svc_run();
   yp_msg("svc_run returned");
